@@ -122,6 +122,7 @@ import (argc, argv)
 #else
 		use_editor = 0;
 #endif
+		if (message) free (message);
 		message = xstrdup(optarg);
 		break;
 	    case 'I':
@@ -219,8 +220,9 @@ import (argc, argv)
      */
     {
 	regex_t pat;
-	assert (!regcomp (&pat, "^[1-9][0-9]*\\.[1-9][0-9]*\\.[1-9][0-9]*$",
-			  REG_EXTENDED));
+	int ret = regcomp (&pat, "^[1-9][0-9]*\\.[1-9][0-9]*\\.[1-9][0-9]*$",
+			   REG_EXTENDED);
+	assert (!ret);
 	if (regexec (&pat, vbranch, 0, NULL, 0))
 	{
 	    error (1, 0,
