@@ -15,6 +15,7 @@
  */
 
 #include "cvs.h"
+#include <assert.h>
 #include "getline.h"
 
 /* Printable names for things in the current_parsed_root->method enum variable.
@@ -376,6 +377,8 @@ parse_cvsroot (root_in)
     int check_hostname, no_port, no_password;
 #endif /* CLIENT_SUPPORT */
 
+    assert (root_in);
+
     /* allocate some space */
     newroot = new_cvsroot_t();
 
@@ -743,6 +746,8 @@ normalize_cvsroot (root)
     char *p, *hostname, *username;
     char port_s[64];
 
+    assert (root && root->hostname && root->directory);
+
     /* get the appropriate port string */
     sprintf (port_s, "%d", get_cvs_port_number (root));
 
